@@ -2,6 +2,7 @@
 A package for generating various graphs in networkx. 
 
 """
+import networkx as nx
 from bipartite import *
 from classic import *
 from degree_seq import *
@@ -12,3 +13,6 @@ from random_graphs import *
 from small import *
 from threshold import *
 from atlas import *
+from line import *
+from ego import *
+from stochastic import *
