@@ -3,46 +3,59 @@
 Generators and functions for bipartite graphs.
 
 """
-#    Copyright (C) 2006 by 
+#    Copyright (C) 2006-2008 by 
 #    Aric Hagberg <hagberg@lanl.gov>
 #    Dan Schult <dschult@colgate.edu>
 #    Pieter Swart <swart@lanl.gov>
-#    Distributed under the terms of the GNU Lesser General Public License
-#    http://www.gnu.org/copyleft/lesser.html
+#    All rights reserved.
+#    BSD license.
 __author__ = """Aric Hagberg (hagberg@lanl.gov)\nPieter Swart (swart@lanl.gov)\nDan Schult (dschult@colgate.edu)"""
  
+__all__=['bipartite_configuration_model',
+         'bipartite_havel_hakimi_graph',
+         'bipartite_reverse_havel_hakimi_graph',
+         'bipartite_alternating_havel_hakimi_graph',
+         'bipartite_preferential_attachment_graph',
+         'bipartite_random_regular_graph',
+         ]
+
 import random
 import sys
-import networkx as NX
+import networkx 
 
-def bipartite_configuration_model(aseq, bseq,
-                                  create_using=None,
-                                  seed=None,
-                                  ):
-    """
-    Return a random bipartite graph from two given degree sequences.
+def bipartite_configuration_model(aseq, bseq, create_using=None, seed=None):
+    """Return a random bipartite graph from two given degree sequences.
 
-    :Parameters:
-       - `aseq`: degree sequence for node set A
-       - `bseq`: degree sequence for node set B
+    Parameters
+    ----------
+    aseq : list or iterator
+       Degree sequence for node set A.
+    bseq : list or iterator
+       Degree sequence for node set B.
+    create_using : NetworkX graph instance, optional
+       Return graph of this type.
+    seed : integer, optional
+       Seed for random number generator. 
 
     Nodes from the set A are connected to nodes in the set B by
     choosing randomly from the possible free stubs, one in A and
     one in B.
 
+    Notes
+    -----
     The sum of the two sequences must be equal: sum(aseq)=sum(bseq)
-
-    If no graph type is specified use XGraph with parallel edges.
-
+    If no graph type is specified use MultiGraph with parallel edges.
     If you want a graph with no parallel edges use create_using=Graph()
     but then the resulting degree sequences might not be exact.
-
     """
+    if create_using is None:
+        create_using=networkx.MultiGraph()
+    elif create_using.is_directed():
+        raise networkx.NetworkXError(\
+                "Directed Graph not supported")
+        
 
-    if create_using==None:
-        create_using=NX.XGraph(multiedges=True)
-
-    G=NX.empty_graph(0,create_using)
+    G=networkx.empty_graph(0,create_using)
 
     if not seed is None:
         random.seed(seed)    
@@ -54,9 +67,9 @@ def bipartite_configuration_model(aseq, bseq,
     sumb=sum(bseq)
 
     if not suma==sumb:
-        raise NX.NetworkXError, \
+        raise networkx.NetworkXError(\
               'invalid degree sequences, sum(aseq)!=sum(bseq),%s,%s'\
-              %(suma,sumb)
+              %(suma,sumb))
 
     G.add_nodes_from(range(0,lena+lenb))
 
@@ -83,27 +96,37 @@ def bipartite_configuration_model(aseq, bseq,
     return G
 
 
-def bipartite_havel_hakimi_graph(aseq, bseq,
-                                 create_using=None,
-                                 ):
-    """
-    Return a bipartite graph from two given degree sequences
+def bipartite_havel_hakimi_graph(aseq, bseq, create_using=None):
+    """Return a bipartite graph from two given degree sequences
     using a Havel-Hakimi style construction.
 
-    :Parameters:
-       - `aseq`: degree sequence for node set A
-       - `bseq`: degree sequence for node set B
+    Parameters
+    ----------
+    aseq : list or iterator
+       Degree sequence for node set A.
+    bseq : list or iterator
+       Degree sequence for node set B.
+    create_using : NetworkX graph instance, optional
+       Return graph of this type.
 
     Nodes from the set A are connected to nodes in the set B by
     connecting the highest degree nodes in set A to
     the highest degree nodes in set B until all stubs are connected.
 
+    Notes
+    -----
     The sum of the two sequences must be equal: sum(aseq)=sum(bseq)
+    If no graph type is specified use MultiGraph with parallel edges.
+    If you want a graph with no parallel edges use create_using=Graph()
+    but then the resulting degree sequences might not be exact.
     """
-    if create_using==None:
-        create_using=NX.XGraph(multiedges=True)
+    if create_using is None:
+        create_using=networkx.MultiGraph()
+    elif create_using.is_directed():
+        raise networkx.NetworkXError(\
+                "Directed Graph not supported")
 
-    G=NX.empty_graph(0,create_using)
+    G=networkx.empty_graph(0,create_using)
 
     # length of the each sequence
     naseq=len(aseq)
@@ -113,9 +136,9 @@ def bipartite_havel_hakimi_graph(aseq, bseq,
     sumb=sum(bseq)
 
     if not suma==sumb:
-        raise NX.NetworkXError, \
+        raise networkx.NetworkXError(\
               'invalid degree sequences, sum(aseq)!=sum(bseq),%s,%s'\
-              %(suma,sumb)
+              %(suma,sumb))
 
     G.add_nodes_from(range(0,naseq)) # one vertex type (a)
     G.add_nodes_from(range(naseq,naseq+nbseq)) # the other type (b)
@@ -141,27 +164,37 @@ def bipartite_havel_hakimi_graph(aseq, bseq,
     G.name="bipartite_havel_hakimi_graph"
     return G
 
-def bipartite_reverse_havel_hakimi_graph(aseq, bseq,
-                                         create_using=None,
-                                         ):
-    """
-    Return a bipartite graph from two given degree sequences
-    using a "reverse" Havel-Hakimi style construction.
+def bipartite_reverse_havel_hakimi_graph(aseq, bseq, create_using=None):
+    """Return a bipartite graph from two given degree sequences
+    using a Havel-Hakimi style construction.
 
-    :Parameters:
-       - `aseq`: degree sequence for node set A
-       - `bseq`: degree sequence for node set B
+    Parameters
+    ----------
+    aseq : list or iterator
+       Degree sequence for node set A.
+    bseq : list or iterator
+       Degree sequence for node set B.
+    create_using : NetworkX graph instance, optional
+       Return graph of this type.
 
     Nodes from the set A are connected to nodes in the set B by
     connecting the highest degree nodes in set A to
     the lowest degree nodes in set B until all stubs are connected.
 
+    Notes
+    -----
     The sum of the two sequences must be equal: sum(aseq)=sum(bseq)
+    If no graph type is specified use MultiGraph with parallel edges.
+    If you want a graph with no parallel edges use create_using=Graph()
+    but then the resulting degree sequences might not be exact.
     """
-    if create_using==None:
-        create_using=NX.XGraph(multiedges=True)
+    if create_using is None:
+        create_using=networkx.MultiGraph()
+    elif create_using.is_directed():
+        raise networkx.NetworkXError(\
+                "Directed Graph not supported")
 
-    G=NX.empty_graph(0,create_using)
+    G=networkx.empty_graph(0,create_using)
 
 
     # length of the each sequence
@@ -171,9 +204,9 @@ def bipartite_reverse_havel_hakimi_graph(aseq, bseq,
     sumb=sum(bseq)
 
     if not suma==sumb:
-        raise NX.NetworkXError, \
+        raise networkx.NetworkXError(\
               'invalid degree sequences, sum(aseq)!=sum(bseq),%s,%s'\
-              %(suma,sumb)
+              %(suma,sumb))
 
     G.add_nodes_from(range(0,lena+lenb))
     
@@ -200,28 +233,38 @@ def bipartite_reverse_havel_hakimi_graph(aseq, bseq,
     return G
 
 
-def bipartite_alternating_havel_hakimi_graph(aseq, bseq,
-                                            create_using=None,
-                                            ):
-    """
-    Return a bipartite graph from two given degree sequences
+def bipartite_alternating_havel_hakimi_graph(aseq, bseq,create_using=None):
+    """Return a bipartite graph from two given degree sequences
     using a alternating Havel-Hakimi style construction.
 
-    :Parameters:
-       - `aseq`: degree sequence for node set A
-       - `bseq`: degree sequence for node set B
+    Parameters
+    ----------
+    aseq : list or iterator
+       Degree sequence for node set A.
+    bseq : list or iterator
+       Degree sequence for node set B.
+    create_using : NetworkX graph instance, optional
+       Return graph of this type.
 
     Nodes from the set A are connected to nodes in the set B by
     connecting the highest degree nodes in set A to
     alternatively the highest and the lowest degree nodes in set
     B until all stubs are connected.
 
+    Notes
+    -----
     The sum of the two sequences must be equal: sum(aseq)=sum(bseq)
+    If no graph type is specified use MultiGraph with parallel edges.
+    If you want a graph with no parallel edges use create_using=Graph()
+    but then the resulting degree sequences might not be exact.
     """
-    if create_using==None:
-        create_using=NX.XGraph(multiedges=True)
+    if create_using is None:
+        create_using=networkx.MultiGraph()
+    elif create_using.is_directed():
+        raise networkx.NetworkXError(\
+                "Directed Graph not supported")
 
-    G=NX.empty_graph(0,create_using)
+    G=networkx.empty_graph(0,create_using)
 
     # length of the each sequence
     naseq=len(aseq)
@@ -230,9 +273,9 @@ def bipartite_alternating_havel_hakimi_graph(aseq, bseq,
     sumb=sum(bseq)
 
     if not suma==sumb:
-        raise NX.NetworkXError, \
+        raise networkx.NetworkXError(\
               'invalid degree sequences, sum(aseq)!=sum(bseq),%s,%s'\
-              %(suma,sumb)
+              %(suma,sumb))
 
     G.add_nodes_from(range(0,naseq)) # one vertex type (a)
     G.add_nodes_from(range(naseq,naseq+nbseq)) # the other type (b)
@@ -261,19 +304,23 @@ def bipartite_alternating_havel_hakimi_graph(aseq, bseq,
     G.name="bipartite_alternating_havel_hakimi_graph"
     return G
 
-def bipartite_preferential_attachment_graph(aseq,p,
-                                            create_using=None,
-                                            ):
-    """
-    Create a bipartite graph with a preferential attachment model
-    from a given single "top" degree sequence.
+def bipartite_preferential_attachment_graph(aseq,p,create_using=None,seed=None):
+    """Create a bipartite graph with a preferential attachment model
+    from a given single degree sequence.
 
-    :Parameters:
-       - `aseq`: degree sequence for node set A (top)
-       - `p`:    probability that a new bottom node is added
+    Parameters
+    ----------
+    aseq : list or iterator
+       Degree sequence for node set A.
+    p :  float
+       Probability that a new bottom node is added.
+    create_using : NetworkX graph instance, optional
+       Return graph of this type.
+    seed : integer, optional
+       Seed for random number generator. 
 
-
-    Reference::
+    Notes
+    -----
 
      @article{guillaume-2004-bipartite,
        author = {Jean-Loup Guillaume and Matthieu Latapy},
@@ -290,13 +337,19 @@ def bipartite_preferential_attachment_graph(aseq,p,
        }
 
     """
-    if create_using==None:
-        create_using=NX.XGraph(multiedges=True)
-
-    G=NX.empty_graph(0,create_using)
+    if create_using is None:
+        create_using=networkx.MultiGraph()
+    elif create_using.is_directed():
+        raise networkx.NetworkXError(\
+                "Directed Graph not supported")
 
     if p > 1: 
-        raise NX.NetworkXError, "probability %s > 1"%(p)
+        raise networkx.NetworkXError("probability %s > 1"%(p))
+
+    G=networkx.empty_graph(0,create_using)
+
+    if not seed is None:
+        random.seed(seed)    
 
     naseq=len(aseq)
     G.add_nodes_from(range(0,naseq))
@@ -320,21 +373,32 @@ def bipartite_preferential_attachment_graph(aseq,p,
     return G
 
 
-def bipartite_random_regular_graph(d, n,
-                                   create_using=None,
-                                   ):
-    """
-    UNTESTED:Generate a random bipartite graph of n nodes each with degree d.
+def bipartite_random_regular_graph(d, n, create_using=None,seed=None):
+    """UNTESTED: Generate a random bipartite graph.
+
+    Parameters
+    ----------
+    d : integer
+      Degree of graph.
+    n : integer
+      Number of nodes in graph.
+    create_using : NetworkX graph instance, optional
+      Return graph of this type.
+    seed : integer, optional
+       Seed for random number generator. 
+
+    Notes
+    ------
+    Nodes are numbered 0...n-1. 
 
     Restrictions on n and d:
        -  n must be even
        -  n>=2*d
 
-    Nodes are numbered 0...n-1. 
-    
     Algorithm inspired by random_regular_graph()
 
     """
+    # This algorithm could be improved - see random_regular_graph()
     # helper subroutine to check for suitable edges
     def suitable(leftstubs,rightstubs):
         for s in leftstubs:
@@ -358,10 +422,16 @@ def bipartite_random_regular_graph(d, n,
         return False
 
 
-    if create_using==None:
-        create_using=NX.XGraph(multiedges=True)
+    if create_using is None:
+        create_using=networkx.MultiGraph()
+    elif create_using.is_directed():
+        raise networkx.NetworkXError(\
+                "Directed Graph not supported")
 
-    G=NX.empty_graph(0,create_using)
+    if not seed is None:
+        random.seed(seed)    
+
+    G=networkx.empty_graph(0,create_using)
 
     nodes=range(0,n)
     G.add_nodes_from(nodes)
@@ -390,81 +460,3 @@ def bipartite_random_regular_graph(d, n,
                return False
     return G
 
-
-def project(B,nodes,create_using=None):
-    """
-    Returns a graph that is the projection of the bipartite graph B
-    onto the set of nodes given in list nodes.
-    
-    The nodes retain their names and are connected if they share a
-    common node in the node set of {B not nodes }.
- 
-    No attempt is made to verify that the input graph B is bipartite.
-    """
-
-    if create_using==None:
-        create_using=NX.Graph()
-
-    G=NX.empty_graph(0,create_using)
-
-    for v in nodes:
-        G.add_node(v)
-        for cv in B.neighbors(v):
-            G.add_edges_from([(v,u) for u in B.neighbors(cv)])
-    return G
-
-
-def bipartite_color(G):
-    color={}
-    for n in G.nodes(): #handle disconnected graphs
-        if n in color: continue
-        queue=[n]  
-        color[n]=1 # nodes seen with color (1 or 0)
-        while queue:
-            v=queue.pop()
-            c=1-color[v] # opposite color of node v
-            for w in G.neighbors(v): 
-                if w in color: 
-                    if color[w]==color[v]:
-                        raise NX.NetworkXError, "graph is not bipartite"
-                else:
-                    color[w]=c
-                    queue.append(w)
-    return color
-
-def is_bipartite(G):
-    """ Returns True if graph G is bipartite, False if not.
-
-    Traverse the graph G with depth-first-search and color nodes.
-    """
-    try:
-        bipartite_color(G)
-        return True
-    except:
-        return False
-    
-def bipartite_sets(G):
-    """
-    Returns (X,Y) where X and Y are the nodes in each bipartite set
-    of graph G.  Fails with an error if graph is not bipartite.
-    """
-    color=bipartite_color(G)
-    X=[n for n in color if color[n]==1]
-    Y=[n for n in color if color[n]==0]
-    return (X,Y)
-
-def _test_suite():
-    import doctest
-    suite = doctest.DocFileSuite('tests/generators/bipartite.txt',
-                                 package='networkx')
-    return suite
-
-if __name__ == "__main__":
-    import sys
-    import unittest
-    if sys.version_info[:2] < (2, 4):
-        print "Python version 2.4 or later required for tests (%d.%d detected)."%  sys.version_info[:2]
-        sys.exit(-1)
-    unittest.TextTestRunner().run(_test_suite())
-    
-
