@@ -3,32 +3,50 @@ Generators for some classic graphs.
 
 The typical graph generator is called as follows:
 
->>> G=complete_graph(100)
+>>> G=nx.complete_graph(100)
 
 returning the complete graph on n nodes labeled 0,..,99
 as a simple graph. Except for empty_graph, all the generators 
 in this module return a Graph class (i.e. a simple, undirected graph).
 
 """
-#    Copyright (C) 2004,2005 by 
+#    Copyright (C) 2004-2008 by 
 #    Aric Hagberg <hagberg@lanl.gov>
 #    Dan Schult <dschult@colgate.edu>
 #    Pieter Swart <swart@lanl.gov>
-#    Distributed under the terms of the GNU Lesser General Public License
-#    http://www.gnu.org/copyleft/lesser.html
+#    All rights reserved.
+#    BSD license.
 __author__ ="""Aric Hagberg (hagberg@lanl.gov)\nPieter Swart (swart@lanl.gov)"""
-__date__ = "$Date: 2005-06-17 14:06:03 -0600 (Fri, 17 Jun 2005) $"
-__credits__ = """"""
-__revision__ = "$Revision: 1056 $"
+
+
+__all__ = [ 'balanced_tree',
+            'barbell_graph',
+            'complete_graph',
+            'complete_bipartite_graph',
+            'circular_ladder_graph',
+            'cycle_graph',
+            'dorogovtsev_goltsev_mendes_graph',
+            'empty_graph',
+            'grid_graph',
+            'grid_2d_graph',
+            'hypercube_graph',
+            'ladder_graph',
+            'lollipop_graph',
+            'null_graph',
+            'path_graph',
+            'star_graph',
+            'trivial_graph',
+            'wheel_graph']
 
 
 #-------------------------------------------------------------------
 #   Some Classic Graphs
 #-------------------------------------------------------------------
-import networkx
+import networkx as nx
+from networkx import utils
 
 
-def balanced_tree(r,h):
+def balanced_tree(r,h,create_using=None):
     """Return the perfectly balanced r-tree of height h.
 
     For r>=2, h>=1, this is the rooted tree where all leaves
@@ -43,13 +61,15 @@ def balanced_tree(r,h):
     
     """
     if r<2:
-        raise networkx.NetworkXError, \
-              "Invalid graph description, r should be >=2"
+        raise nx.NetworkXError(\
+              "Invalid graph description, r should be >=2")
     if h<1:
-        raise networkx.NetworkXError, \
-              "Invalid graph description, h should be >=1"
+        raise nx.NetworkXError(\
+              "Invalid graph description, h should be >=1")
     
-    G=empty_graph()
+    if create_using is not None and create_using.is_directed():
+        raise nx.NetworkXError("Directed Graph not supported")
+    G=empty_graph(0,create_using)
     G.name="balanced_tree(%d,%d)"%(r,h)
 
     # Grow tree of increasing height by repeatedly adding a layer
@@ -79,7 +99,7 @@ def balanced_tree(r,h):
         height=height+1
     return G
 
-def barbell_graph(m1,m2):
+def barbell_graph(m1,m2,create_using=None):
     """Return the Barbell Graph: two complete graphs connected by a path.
 
     For m1 > 1 and m2 >= 0.
@@ -100,14 +120,14 @@ def barbell_graph(m1,m2):
 
     """
     if m1<2:
-        raise networkx.NetworkXError, \
-              "Invalid graph description, m1 should be >=2"
+        raise nx.NetworkXError(\
+              "Invalid graph description, m1 should be >=2")
     if m2<0:
-        raise networkx.NetworkXError, \
-              "Invalid graph description, m2 should be >=0"
+        raise nx.NetworkXError(\
+              "Invalid graph description, m2 should be >=0")
 
     # left barbell
-    G=complete_graph(m1)
+    G=complete_graph(m1,create_using)
     G.name="barbell_graph(%d,%d)"%(m1,m2)
     
     # connecting path
@@ -115,9 +135,7 @@ def barbell_graph(m1,m2):
     if m2>1:
         G.add_edges_from([(v,v+1) for v in range(m1,m1+m2-1)])
     # right barbell
-    for u in range(m1+m2,2*m1+m2):
-        for v in range(u,2*m1+m2):
-            G.add_edge(u,v)
+    G.add_edges_from( (u,v) for u in range(m1+m2,2*m1+m2) for v in range(u+1,2*m1+m2))
     # connect it up
     G.add_edge(m1-1,m1)
     if m2>0:
@@ -130,14 +148,16 @@ def complete_graph(n,create_using=None):
     Node labels are the integers 0 to n-1.
 
     """
+    if create_using is not None and create_using.is_directed():
+        raise nx.NetworkXError("Directed Graph not supported")
     G=empty_graph(n,create_using)
     G.name="complete_graph(%d)"%n
     for u in xrange(n):
-        for v in xrange(u,n):
+        for v in xrange(u+1,n):
             G.add_edge(u,v)
     return G
 
-def complete_bipartite_graph(n1,n2):
+def complete_bipartite_graph(n1,n2,create_using=None):
     """Return the complete bipartite graph K_{n1_n2}.
     
     Composed of two partitions with n1 nodes in the first 
@@ -147,14 +167,16 @@ def complete_bipartite_graph(n1,n2):
     Node labels are the integers 0 to n1+n2-1
     
     """
-    G=empty_graph(n1+n2)
+    if create_using is not None and create_using.is_directed():
+        raise nx.NetworkXError("Directed Graph not supported")
+    G=empty_graph(n1+n2,create_using)
     G.name="complete_bipartite_graph(%d,%d)"%(n1,n2)
     for v1 in xrange(n1):
         for v2 in xrange(n2):
             G.add_edge(v1,n1+v2)
     return G
 
-def circular_ladder_graph(n):
+def circular_ladder_graph(n,create_using=None):
     """Return the circular ladder graph CL_n of length n.
 
     CL_n consists of two concentric n-cycles in which
@@ -163,7 +185,7 @@ def circular_ladder_graph(n):
     Node labels are the integers 0 to n-1
     
     """
-    G=ladder_graph(n)
+    G=ladder_graph(n,create_using)
     G.name="circular_ladder_graph(%d)"%n
     G.add_edge(0,n-1)
     G.add_edge(n,2*n-1)
@@ -183,14 +205,19 @@ def cycle_graph(n,create_using=None):
     if n>1: G.add_edge(n-1,0)
     return G
 
-def dorogovtsev_goltsev_mendes_graph(n):
+def dorogovtsev_goltsev_mendes_graph(n,create_using=None):
     """Return the hierarchically constructed Dorogovtsev-Goltsev-Mendes graph.
 
     n is the generation.
     See: arXiv:/cond-mat/0112143 by Dorogovtsev, Goltsev and Mendes.
     
     """
-    G=networkx.Graph()
+    if create_using is not None:
+        if create_using.is_directed():
+            raise nx.NetworkXError("Directed Graph not supported")
+        if create_using.is_multigraph():
+            raise nx.NetworkXError("Multigraph not supported")
+    G=empty_graph(0,create_using)
     G.name="Dorogovtsev-Goltsev-Mendes Graph"
     G.add_edge(0,1)
     if n==0:
@@ -211,8 +238,7 @@ def empty_graph(n=0,create_using=None):
     Node labels are the integers 0 to n-1
 
     For example:
-    >>> from networkx import *
-    >>> G=empty_graph(10)
+    >>> G=nx.empty_graph(10)
     >>> G.number_of_nodes()
     10
     >>> G.number_of_edges()
@@ -229,7 +255,7 @@ def empty_graph(n=0,create_using=None):
     empty digraph, network,etc.  For example,
 
     >>> n=10
-    >>> G=empty_graph(n,create_using=DiGraph())
+    >>> G=nx.empty_graph(n,create_using=nx.DiGraph())
 
     will create an empty digraph on n nodes.
 
@@ -245,7 +271,7 @@ def empty_graph(n=0,create_using=None):
     """
     if create_using is None:
         # default empty graph is a simple graph
-        G=networkx.Graph()
+        G=nx.Graph()
     else:
         G=create_using
         G.clear()
@@ -254,37 +280,36 @@ def empty_graph(n=0,create_using=None):
     G.name="empty_graph(%d)"%n 
     return G
 
-def grid_2d_graph(m,n,periodic=False):
+def grid_2d_graph(m,n,periodic=False,create_using=None):
     """ Return the 2d grid graph of mxn nodes,
         each connected to its nearest neighbors.
         Optional argument periodic=True will connect
         boundary nodes via periodic boundary conditions.
     """
-    G=empty_graph()
-    G.name="grid 2d graph"
+    G=empty_graph(0,create_using)
+    G.name="grid_2d_graph"
     rows=range(m)
     columns=range(n)
-    def _label(i,j):
-        return (i,j)
-    for i in rows:
-        for j in columns:
-            G.add_node( _label(i,j) )
-    for i in rows:
-        for j in columns:
-            if i>0: G.add_edge( _label(i,j), _label(i-1,j) )
-            if i<m-1: G.add_edge( _label(i,j), _label(i+1,j) )
-            if j>0: G.add_edge( _label(i,j), _label(i,j-1) )
-            if j<n-1: G.add_edge( _label(i,j), _label(i,j+1) )
+    G.add_nodes_from( (i,j) for i in rows for j in columns )
+    G.add_edges_from( ((i,j),(i-1,j)) for i in rows for j in columns if i>0 )
+    G.add_edges_from( ((i,j),(i,j-1)) for i in rows for j in columns if j>0 )
+    if G.is_directed():
+        G.add_edges_from( ((i,j),(i+1,j)) for i in rows for j in columns if i<m-1 )
+        G.add_edges_from( ((i,j),(i,j+1)) for i in rows for j in columns if j<n-1 )
     if periodic:
-        for i in rows:
-            G.add_edge(_label(i,0), _label(i,n-1))
-        for j in columns:
-            G.add_edge(_label(0,j), _label(m-1,j))
+        if n>2:
+            G.add_edges_from( ((i,0),(i,n-1)) for i in rows )
+            if G.is_directed():
+                G.add_edges_from( ((i,n-1),(i,0)) for i in rows )
+        if m>2:
+            G.add_edges_from( ((0,j),(m-1,j)) for j in columns )
+            if G.is_directed():
+                G.add_edges_from( ((m-1,j),(0,j)) for j in columns )
         G.name="periodic_grid_2d_graph(%d,%d)"%(m,n)
     return G
 
 
-def grid_graph(dim,periodic=False):
+def grid_graph(dim,periodic=False,create_using=None):
     """ Return the n-dimensional grid graph.
 
     The dimension is the length of the list 'dim' and the
@@ -296,46 +321,51 @@ def grid_graph(dim,periodic=False):
 
     """    
     from networkx.utils import is_list_of_ints
+    if create_using is not None and create_using.is_directed():
+        raise nx.NetworkXError("Directed Graph not supported")
     dlabel="%s"%dim
     if dim==[]:
-        G=networkx.Graph()
+        G=empty_graph(0,create_using)
         G.name="grid_graph(%s)"%dim
         return G
     if not is_list_of_ints(dim):
-        raise networkx.NetworkXError,"dim is not a list of integers"
+        raise nx.NetworkXError("dim is not a list of integers")
     if min(dim)<=0:
-        raise networkx.NetworkXError,\
-              "dim is not a list of strictly positive integers"       
+        raise nx.NetworkXError(\
+              "dim is not a list of strictly positive integers")      
     if periodic:
         func=cycle_graph
     else:
         func=path_graph
 
     current_dim=dim.pop()
-    G=func(current_dim)
+    G=func(current_dim,create_using)
     while len(dim)>0:
         current_dim=dim.pop() 
-        Gnew=func(current_dim)
-        Gold=G.copy()
-        G=networkx.operators.cartesian_product(Gnew,Gold)
+        # order matters: copy before it is cleared during the creation of Gnew
+        Gold=G.copy() 
+        Gnew=func(current_dim,create_using)
+        # explicit: create_using=None 
+        # This is so that we get a new graph of Gnew's class.
+        G=nx.cartesian_product(Gnew,Gold,create_using=None)
     # graph G is done but has labels of the form (1,(2,(3,1)))
     # so relabel
-    H=networkx.operators.relabel_nodes(G, networkx.utils.flatten)
+    H=nx.relabel_nodes(G, utils.flatten)
     H.name="grid_graph(%s)"%dlabel
     return H
 
-def hypercube_graph(n):
+def hypercube_graph(n,create_using=None):
     """Return the n-dimensional hypercube.
 
     Node labels are the integers 0 to 2**n - 1.
 
     """
     dim=n*[2]
-    G=grid_graph(dim)
+    G=grid_graph(dim,create_using=create_using)
     G.name="hypercube_graph_(%d)"%n
     return G
 
-def ladder_graph(n):
+def ladder_graph(n,create_using=None):
     """Return the Ladder graph of length n.
 
     This is two rows of n nodes, with
@@ -344,14 +374,16 @@ def ladder_graph(n):
     Node labels are the integers 0 to 2*n - 1.
     
     """
-    G=empty_graph(2*n)
+    if create_using is not None and create_using.is_directed():
+        raise nx.NetworkXError("Directed Graph not supported")
+    G=empty_graph(2*n,create_using)
     G.name="ladder_graph_(%d)"%n 
     G.add_edges_from([(v,v+1) for v in xrange(n-1)])
     G.add_edges_from([(v,v+1) for v in xrange(n,2*n-1)])
     G.add_edges_from([(v,v+n) for v in xrange(n)])
     return G
 
-def lollipop_graph(m,n):
+def lollipop_graph(m,n,create_using=None):
     """Return the Lollipop Graph; K_m connected to P_n.
 
     This is the Barbell Graph without the right barbell.
@@ -369,13 +401,13 @@ def lollipop_graph(m,n):
     
     """
     if m<2:
-        raise networkx.NetworkXError, \
-              "Invalid graph description, m should be >=2"
+        raise nx.NetworkXError(\
+              "Invalid graph description, m should be >=2")
     if n<0:
-        raise networkx.NetworkXError, \
-              "Invalid graph description, n should be >=0"
+        raise nx.NetworkXError(\
+              "Invalid graph description, n should be >=0")
     # the ball
-    G=complete_graph(m)
+    G=complete_graph(m,create_using)
     # the stick
     G.add_nodes_from([v for v in xrange(m,m+n)])
     if n>1:
@@ -409,58 +441,37 @@ def path_graph(n,create_using=None):
     G.add_edges_from([(v,v+1) for v in xrange(n-1)])
     return G
 
-def star_graph(n):
+def star_graph(n,create_using=None):
     """ Return the Star graph with n+1 nodes:
     one center node, connected to n outer nodes.
 
    Node labels are the integers 0 to n.
 
     """
-    G=complete_bipartite_graph(1,n)
+    G=complete_bipartite_graph(1,n,create_using)
     G.name="star_graph(%d)"%n
     return G
 
-def trivial_graph():
+def trivial_graph(create_using=None):
     """ Return the Trivial graph with one node (with integer label 0)
     and no edges.
 
     """
-    G=empty_graph(1)
+    G=empty_graph(1,create_using)
     G.name="trivial_graph()"
     return G
 
-def wheel_graph(n):
+def wheel_graph(n,create_using=None):
     """ Return the wheel graph: a single hub node connected
     to each node of the (n-1)-node cycle graph. 
 
    Node labels are the integers 0 to n - 1.
 
     """
-    G=star_graph(n-1)
+    G=star_graph(n-1,create_using)
     G.name="wheel_graph(%d)"%n
     G.add_edges_from([(v,v+1) for v in xrange(1,n-1)])
-    if n>1:
+    if n>2:
         G.add_edge(1,n-1)
     return G
-                        
 
-def _test_suite():
-    import doctest
-    suite = doctest.DocFileSuite('tests/generators/classic.txt',
-                                 package='networkx')
-    return suite
-
-
-if __name__ == "__main__":
-    import os
-    import sys
-    import unittest
-    if sys.version_info[:2] < (2, 4):
-        print "Python version 2.4 or later required (%d.%d detected)."\
-              %  sys.version_info[:2]
-        sys.exit(-1)
-    # directory of networkx package (relative to this)
-    nxbase=sys.path[0]+os.sep+os.pardir
-    sys.path.insert(0,nxbase) # prepend to search path
-    unittest.TextTestRunner().run(_test_suite())
-    
