@@ -2,30 +2,34 @@
 Generate graphs with a given degree sequence or expected degree sequence.
 
 """
-#    Copyright (C) 2004-2008 by 
+#    Copyright (C) 2004-2009 by 
 #    Aric Hagberg <hagberg@lanl.gov>
 #    Dan Schult <dschult@colgate.edu>
 #    Pieter Swart <swart@lanl.gov>
-#    Distributed under the terms of the GNU Lesser General Public License
-#    http://www.gnu.org/copyleft/lesser.html
+#    All rights reserved.
+#    BSD license.
 
-__author__ = """Aric Hagberg (hagberg@lanl.gov)\nPieter Swart (swart@lanl.gov)\nDan Schult (dschult@colgate.edu)"""
+__author__ = "\n".join(['Aric Hagberg (hagberg@lanl.gov)',
+                        'Pieter Swart (swart@lanl.gov)',
+                        'Dan Schult (dschult@colgate.edu)'
+                        'Joel Miller (joel.c.miller.research@gmail.com)'])
 
 
 __all__ = ['configuration_model',
+           'directed_configuration_model',
            'expected_degree_graph',
            'havel_hakimi_graph',
            'degree_sequence_tree',
+           'random_clustered_graph',
            'is_valid_degree_sequence',
            'create_degree_sequence',
            'double_edge_swap',
            'connected_double_edge_swap',
-           'li_smax_graph',
-           's_metric']
+           'li_smax_graph']
 
 
 import random
-import networkx
+import networkx as nx
 import networkx.utils
 from networkx.generators.classic import empty_graph
 import heapq
@@ -33,63 +37,88 @@ import heapq
 #  Generating Graphs with a given degree sequence
 #---------------------------------------------------------------------------
 
-def configuration_model(deg_sequence,seed=None):
-    """Return a random pseudograph with the given degree sequence.
+def configuration_model(deg_sequence,create_using=None,seed=None):
+    """Return a random graph with the given degree sequence.
 
-      - `deg_sequence`: degree sequence, a list of integers with each entry
-                        corresponding to the degree of a node (need not be
-                        sorted). A non-graphical degree sequence (i.e. one
-                        not realizable by some simple graph) will raise an
-                        Exception.
-      - `seed`: seed for random number generator (default=None)
+    The configuration model generates a random pseudograph (graph with
+    parallel edges and self loops) by randomly assigning edges to
+    match the given degree sequence.
 
+    Parameters
+    ----------
+    deg_sequence :  list of integers 
+        Each list entry corresponds to the degree of a node.
+    create_using : graph, optional (default MultiGraph)
+       Return graph of this type. The instance will be cleared.
+    seed : hashable object, optional
+        Seed for random number generator.   
 
+    Returns
+    -------
+    G : MultiGraph
+        A graph with the specified degree sequence.
+        Nodes are labeled starting at 0 with an index
+        corresponding to the position in deg_sequence.
+
+    Raises
+    ------
+    NetworkXError
+        If the degree sequence does not have an even sum.
+
+    See Also
+    --------
+    is_valid_degree_sequence
+    
+    Notes
+    -----
+    As described by Newman [1]_.
+
+    A non-graphical degree sequence (not realizable by some simple
+    graph) is allowed since this function returns graphs with self
+    loops and parallel edges.  An exception is raised if the degree
+    sequence does not have an even sum.
+
+    This configuration model construction process can lead to
+    duplicate edges and loops.  You can remove the self-loops and
+    parallel edges (see below) which will likely result in a graph
+    that doesn't have the exact degree sequence specified.  This
+    "finite-size effect" decreases as the size of the graph increases.
+
+    References
+    ----------
+    .. [1] M.E.J. Newman, "The structure and function
+           of complex networks", SIAM REVIEW 45-2, pp 167-256, 2003.
+        
+    Examples
+    --------
     >>> from networkx.utils import powerlaw_sequence
     >>> z=nx.create_degree_sequence(100,powerlaw_sequence)
     >>> G=nx.configuration_model(z)
 
-    The pseudograph G is a networkx.MultiGraph that allows multiple (parallel) 
-    edges between nodes and self-loops (edges from a node to itself).
-
     To remove parallel edges:
 
     >>> G=nx.Graph(G)
 
-    Steps:
-
-     - Check if deg_sequence is a valid degree sequence.
-     - Create N nodes with stubs for attaching edges
-     - Randomly select two available stubs and connect them with an edge.
-
-    As described by Newman [newman-2003-structure].
+    To remove self loops:
     
-    Nodes are labeled 1,.., len(deg_sequence),
-    corresponding to their position in deg_sequence.
-
-    This process can lead to duplicate edges and loops, and therefore
-    returns a pseudograph type.  You can remove the self-loops and
-    parallel edges (see above) with the likely result of
-    not getting the exat degree sequence specified.
-    This "finite-size effect" decreases as the size of the graph increases.
-
-    References:
-    
-    [newman-2003-structure]  M.E.J. Newman, "The structure and function
-    of complex networks", SIAM REVIEW 45-2, pp 167-256, 2003.
-        
+    >>> G.remove_edges_from(G.selfloop_edges())
     """
     if not sum(deg_sequence)%2 ==0:
-        raise networkx.NetworkXError, 'Invalid degree sequence'
+        raise nx.NetworkXError('Invalid degree sequence')
+
+    if create_using is None:
+        create_using = nx.MultiGraph()
+    elif create_using.is_directed():
+        raise nx.NetworkXError("Directed Graph not supported")
 
     if not seed is None:
         random.seed(seed)
 
     # start with empty N-node graph
     N=len(deg_sequence)
-#    G=networkx.empty_graph(N,create_using=networkx.Graph()) # no multiedges or selfloops
 
     # allow multiedges and selfloops
-    G=networkx.empty_graph(N,create_using=networkx.MultiGraph())
+    G=nx.empty_graph(N,create_using)
 
     if N==0 or max(deg_sequence)==0: # done if no edges
         return G 
@@ -100,7 +129,7 @@ def configuration_model(deg_sequence,seed=None):
     # i.e., node 1 has degree=3 and is repeated 3 times, etc.
     stublist=[]
     for n in G:
-        for i in range(deg_sequence[n-1]):
+        for i in range(deg_sequence[n]):
             stublist.append(n)
 
     # shuffle stublist and assign pairs by removing 2 elements at a time      
@@ -114,35 +143,163 @@ def configuration_model(deg_sequence,seed=None):
     return G
 
 
-def expected_degree_graph(w, seed=None):
+def directed_configuration_model(in_degree_sequence,
+                                 out_degree_sequence,
+                                 create_using=None,seed=None):
+    """Return a directed_random graph with the given degree sequences.
+
+    The configuration model generates a random directed pseudograph
+    (graph with parallel edges and self loops) by randomly assigning
+    edges to match the given degree sequences.
+
+    Parameters
+    ----------
+    in_degree_sequence :  list of integers 
+       Each list entry corresponds to the in-degree of a node.
+    out_degree_sequence :  list of integers 
+       Each list entry corresponds to the out-degree of a node.
+    create_using : graph, optional (default MultiDiGraph)
+       Return graph of this type. The instance will be cleared.
+    seed : hashable object, optional
+        Seed for random number generator.   
+
+    Returns
+    -------
+    G : MultiDiGraph
+        A graph with the specified degree sequences.
+        Nodes are labeled starting at 0 with an index
+        corresponding to the position in deg_sequence.
+
+    Raises
+    ------
+    NetworkXError
+        If the degree sequences do not have the same sum.
+
+    See Also
+    --------
+    configuration_model
+    
+    Notes
+    -----
+    Algorithm as described by Newman [1]_.
+
+    A non-graphical degree sequence (not realizable by some simple
+    graph) is allowed since this function returns graphs with self
+    loops and parallel edges.  An exception is raised if the degree
+    sequences does not have the same sum.
+
+    This configuration model construction process can lead to
+    duplicate edges and loops.  You can remove the self-loops and
+    parallel edges (see below) which will likely result in a graph
+    that doesn't have the exact degree sequence specified.  This
+    "finite-size effect" decreases as the size of the graph increases.
+
+    References
+    ----------
+    .. [1] Newman, M. E. J. and Strogatz, S. H. and Watts, D. J.
+       Random graphs with arbitrary degree distributions and their applications
+       Phys. Rev. E, 64, 026118 (2001)
+        
+    Examples
+    --------
+    >>> D=nx.DiGraph([(0,1),(1,2),(2,3)]) # directed path graph
+    >>> din=D.in_degree().values()
+    >>> dout=D.out_degree().values()
+    >>> din.append(1) 
+    >>> dout[0]=2
+    >>> D=nx.directed_configuration_model(din,dout)
+
+    To remove parallel edges:
+
+    >>> D=nx.DiGraph(D)
+
+    To remove self loops:
+    
+    >>> D.remove_edges_from(D.selfloop_edges())
+    """
+    if not sum(in_degree_sequence) == sum(in_degree_sequence):
+        raise nx.NetworkXError(
+            'Invalid degree sequences. Sequences must have equal sums.')
+
+    if create_using is None:
+        create_using = nx.MultiDiGraph()
+
+    if not seed is None:
+        random.seed(seed)
+
+    nin=len(in_degree_sequence)
+    nout=len(out_degree_sequence)
+
+    # pad in- or out-degree sequence with zeros to match lengths
+    if nin>nout:
+        out_degree_sequence.extend((nin-nout)*[0])
+    else:
+        in_degree_sequence.extend((nout-nin)*[0])                             
+
+    # start with empty N-node graph
+    N=len(in_degree_sequence)
+
+    # allow multiedges and selfloops
+    G=nx.empty_graph(N,create_using)
+
+    if N==0 or max(in_degree_sequence)==0: # done if no edges
+        return G 
+
+    # build stublists of available degree-repeated stubs
+    # e.g. for degree_sequence=[3,2,1,1,1]
+    # initially, stublist=[1,1,1,2,2,3,4,5]
+    # i.e., node 1 has degree=3 and is repeated 3 times, etc.
+    in_stublist=[]
+    for n in G:
+        for i in range(in_degree_sequence[n]):
+            in_stublist.append(n)
+
+    out_stublist=[]
+    for n in G:
+        for i in range(out_degree_sequence[n]):
+            out_stublist.append(n)
+
+    # shuffle stublists and assign pairs by removing 2 elements at a time      
+    random.shuffle(in_stublist)
+    random.shuffle(out_stublist)
+    while in_stublist and out_stublist:
+        source = out_stublist.pop()
+        target = in_stublist.pop()
+        G.add_edge(source,target)
+
+    G.name="directed configuration_model %d nodes %d edges"%(G.order(),G.size())
+    return G
+
+
+
+def expected_degree_graph(w, create_using=None, seed=None):
     """Return a random graph G(w) with expected degrees given by w.
 
-    :Parameters:
-       - `w`: a list of expected degrees
-       - `seed`: seed for random number generator (default=None)
+    Parameters
+    ----------
+    w : list 
+        The list of expected degrees.
+    create_using : graph, optional (default Graph)
+        Return graph of this type. The instance will be cleared.
+    seed : hashable object, optional
+        The seed for the random number generator.
 
+    Examples
+    --------
     >>> z=[10 for i in range(100)]
     >>> G=nx.expected_degree_graph(z)
 
-
-    Reference::
-
-      @Article{connected-components-2002,
-        author =	{Fan Chung and L. Lu},
-        title = 	{Connected components in random graphs
-        with given expected degree sequences},
-        journal = 	{Ann. Combinatorics},
-        year = 		{2002},
-        volume = 	{6},
-        pages = 	{125-145},
-        }
-
-
+    References
+    ----------
+    .. [1] Fan Chung and L. Lu,
+        Connected components in random graphs with given expected
+        degree sequences, Ann. Combinatorics, 6, pp. 125-145, 2002.
 	"""
-
     n = len(w)
     # allow self loops
-    G=networkx.empty_graph(n,create_using=networkx.Graph())
+    if create_using is not None and create_using.is_directed():
+        raise nx.NetworkXError("Directed Graph not supported")
+    G=nx.empty_graph(n,create_using)
     G.name="random_expected_degree_graph"
 
     if n==0 or max(w)==0: # done if no edges
@@ -152,9 +309,9 @@ def expected_degree_graph(w, seed=None):
     rho = 1.0 / float(d) # Vol(G)
     for i in xrange(n):
         if (w[i])**2 > d:
-            raise networkx.NetworkXError,\
+            raise nx.NetworkXError(\
                   "NetworkXError w[i]**2 must be <= sum(w)\
-                  for all i, w[%d] = %f, sum(w) = %f" % (i,w[i],d)
+                  for all i, w[%d] = %f, sum(w) = %f" % (i,w[i],d))
 
     if seed is not None:
         random.seed(seed)
@@ -166,15 +323,26 @@ def expected_degree_graph(w, seed=None):
     return G 
 
 
-def havel_hakimi_graph(deg_sequence):
+def havel_hakimi_graph(deg_sequence,create_using=None):
     """Return a simple graph with given degree sequence, constructed using the
     Havel-Hakimi algorithm.
 
-      - `deg_sequence`: degree sequence, a list of integers with each entry
-         corresponding to the degree of a node (need not be sorted).
-         A non-graphical degree sequence (not sorted).
-         A non-graphical degree sequence (i.e. one
-         not realizable by some simple graph) raises an Exception.    
+    Parameters
+    ----------
+    deg_sequence: list of integers
+        Each integer corresponds to the degree of a node (need not be sorted).
+    create_using : graph, optional (default Graph)
+        Return graph of this type. The instance will be cleared.
+        Multigraphs and directed graphs are not allowed.
+
+    Raises
+    ------
+    NetworkXException
+        For a non-graphical degree sequence (i.e. one
+        not realizable by some simple graph).
+
+    Notes
+    -----
 
     The Havel-Hakimi algorithm constructs a simple graph by
     successively connecting the node of highest degree to other nodes
@@ -186,18 +354,21 @@ def havel_hakimi_graph(deg_sequence):
     See Theorem 1.4 in [chartrand-graphs-1996].
     This algorithm is also used in the function is_valid_degree_sequence.
 
-    References:
-
-    [chartrand-graphs-1996] G. Chartrand and L. Lesniak, "Graphs and Digraphs",
-                            Chapman and Hall/CRC, 1996.
-
+    References
+    ----------
+    .. [1] G. Chartrand and L. Lesniak, "Graphs and Digraphs",
+           Chapman and Hall/CRC, 1996.
     """
     if not is_valid_degree_sequence(deg_sequence):
-        raise networkx.NetworkXError, 'Invalid degree sequence'
-
+        raise nx.NetworkXError('Invalid degree sequence')
+    if create_using is not None:
+        if create_using.is_directed():
+            raise nx.NetworkXError("Directed Graph not supported")
+        if create_using.is_multigraph():
+            raise nx.NetworkXError("Havel-Hakimi requires simple graph")
 
     N=len(deg_sequence)
-    G=networkx.empty_graph(N) # always return a simple graph
+    G=nx.empty_graph(N,create_using) 
 
     if N==0 or max(deg_sequence)==0: # done if no edges
         return G 
@@ -223,7 +394,112 @@ def havel_hakimi_graph(deg_sequence):
     G.name="havel_hakimi_graph %d nodes %d edges"%(G.order(),G.size())
     return G
 
-def degree_sequence_tree(deg_sequence):
+def random_clustered_graph(joint_degree_sequence, create_using=None, seed=None):
+    """Generate a random graph with the given joint degree and triangle
+    degree sequence.
+	
+    This uses a configuration model-like approach to generate a
+    random pseudograph (graph with parallel edges and self loops) by
+    randomly assigning edges to match the given indepdenent edge 
+    and triangle degree sequence.
+
+    Parameters 
+    ---------- 
+    joint_degree_sequence : list of integer pairs
+        Each list entry corresponds to the independent edge degree and
+        triangle degree of a node.
+    create_using : graph, optional (default MultiGraph)
+        Return graph of this type. The instance will be cleared.
+    seed : hashable object, optional
+        The seed for the random number generator.
+
+    Returns
+    -------
+    G : MultiGraph
+        A graph with the specified degree sequence. Nodes are labeled
+        starting at 0 with an index corresponding to the position in
+        deg_sequence.
+	
+    Raises
+    ------
+    NetworkXError
+        If the independent edge degree sequence sum is not even
+        or the triangle degree sequence sum is not divisible by 3.
+
+    Notes
+    -----
+    As described by Miller [1] (see also Newman [2] for an equivalent
+    description).
+	
+    A non-graphical degree sequence (not realizable by some simple
+    graph) is allowed since this function returns graphs with self
+    loops and parallel edges.  An exception is raised if the
+    independent degree sequence does not have an even sum or the
+    triangle degree sequence sum is not divisible by 3.
+	
+    This configuration model-like construction process can lead to
+    duplicate edges and loops.  You can remove the self-loops and
+    parallel edges (see below) which will likely result in a graph
+    that doesn't have the exact degree sequence specified.  This
+    "finite-size effect" decreases as the size of the graph increases.
+
+    References
+    ----------
+    .. [1] J. C. Miller "Percolation and Epidemics on Random Clustered Graphs."
+        Physical Review E, Rapid Communication (to appear).
+    .. [2] M.E.J. Newman, "Random clustered networks".
+        Physical Review Letters (to appear).
+	       
+    Examples
+    --------
+    >>> deg_tri=[[1,0],[1,0],[1,0],[2,0],[1,0],[2,1],[0,1],[0,1]]
+    >>> G = nx.random_clustered_graph(deg_tri)
+
+    To remove parallel edges:
+    >>> G=nx.Graph(G)
+	
+    To remove self loops:
+    >>> G.remove_edges_from(G.selfloop_edges())
+
+    """
+    if create_using is None:
+        create_using = nx.MultiGraph()
+    elif create_using.is_directed():
+        raise nx.NetworkXError("Directed Graph not supported")
+
+    if not seed is None:
+        random.seed(seed)
+
+    N = len(joint_degree_sequence)
+    G = nx.empty_graph(N,create_using)
+
+    ilist = []
+    tlist = []
+    for n in G:
+        degrees = joint_degree_sequence[n]
+        for icount in range(degrees[0]):
+            ilist.append(n)
+        for tcount in range(degrees[1]):
+            tlist.append(n)
+
+    if len(ilist)%2 != 0 or len(tlist)%3 != 0:
+        raise nx.NetworkXError('Invalid degree sequence')
+
+    random.shuffle(ilist)
+    random.shuffle(tlist)
+    while ilist:
+        G.add_edge(ilist.pop(),ilist.pop())
+    while tlist:
+        n1 = tlist.pop()
+        n2 = tlist.pop()
+        n3 = tlist.pop()
+        G.add_edges_from([(n1,n2),(n1,n3),(n2,n3)])
+    G.name = "random_clustered %d nodes %d edges"%(G.order(),G.size())
+    return G
+
+
+
+def degree_sequence_tree(deg_sequence,create_using=None):
     """
     Make a tree for the given degree sequence.
 
@@ -233,18 +509,22 @@ def degree_sequence_tree(deg_sequence):
     """
 
     if not len(deg_sequence)-sum(deg_sequence)/2.0 == 1.0:
-        raise networkx.NetworkXError,"Degree sequence invalid"
+        raise nx.NetworkXError("Degree sequence invalid")
+    if create_using is not None and create_using.is_directed():
+        raise nx.NetworkXError("Directed Graph not supported")
 
-    G=empty_graph(0)
     # single node tree
     if len(deg_sequence)==1:
+        G=empty_graph(0,create_using)
         return G
-    deg=[s for s in deg_sequence if s>1] # all degrees greater than 1
+
+    # all degrees greater than 1
+    deg=[s for s in deg_sequence if s>1] 
     deg.sort(reverse=True)
 
     # make path graph as backbone
     n=len(deg)+2
-    G=networkx.path_graph(n)
+    G=nx.path_graph(n,create_using)
     last=n
 
     # add the leaves
@@ -256,7 +536,7 @@ def degree_sequence_tree(deg_sequence):
         
     # in case we added one too many 
     if len(G.degree())>len(deg_sequence): 
-        G.delete_node(0)
+        G.remove_node(0)
     return G
         
 
@@ -269,19 +549,18 @@ def is_valid_degree_sequence(deg_sequence):
          A non-graphical degree sequence (i.e. one not realizable by some
          simple graph) will raise an exception.
                         
-    See Theorem 1.4 in [chartrand-graphs-1996]. This algorithm is also used
+    See Theorem 1.4 in [1]_. This algorithm is also used
     in havel_hakimi_graph()
 
-    References:
-
-    [chartrand-graphs-1996] G. Chartrand and L. Lesniak, "Graphs and Digraphs",
-                            Chapman and Hall/CRC, 1996.
-
+    References
+    ----------
+    .. [1] G. Chartrand and L. Lesniak, "Graphs and Digraphs",
+           Chapman and Hall/CRC, 1996.
     """
     # some simple tests 
     if deg_sequence==[]:
         return True # empty sequence = empty graph 
-    if not networkx.utils.is_list_of_ints(deg_sequence):
+    if not nx.utils.is_list_of_ints(deg_sequence):
         return False   # list of ints
     if min(deg_sequence)<0:
         return False      # each int not negative
@@ -316,12 +595,18 @@ def create_degree_sequence(n, sfunction=None, max_tries=50, **kwds):
     """ Attempt to create a valid degree sequence of length n using
     specified function sfunction(n,**kwds).
 
-      - `n`: length of degree sequence = number of nodes
-      - `sfunction`: a function, called as "sfunction(n,**kwds)",
-         that returns a list of n real or integer values.
-      - `max_tries`: max number of attempts at creating valid degree
-         sequence.
+    Parameters
+    ----------
+    n : int
+        Length of degree sequence = number of nodes
+    sfunction: function
+        Function which returns a list of n real or integer values.
+        Called as "sfunction(n,**kwds)".
+    max_tries: int
+        Max number of attempts at creating valid degree sequence.
 
+    Notes
+    -----
     Repeatedly create a degree sequence by calling sfunction(n,**kwds)
     until achieving a valid degree sequence. If unsuccessful after
     max_tries attempts, raise an exception.
@@ -329,9 +614,10 @@ def create_degree_sequence(n, sfunction=None, max_tries=50, **kwds):
     For examples of sfunctions that return sequences of random numbers,
     see networkx.Utils.
 
+    Examples
+    --------
     >>> from networkx.utils import uniform_sequence
     >>> seq=nx.create_degree_sequence(10,uniform_sequence)
-
     """
     tries=0
     max_deg=n
@@ -343,8 +629,8 @@ def create_degree_sequence(n, sfunction=None, max_tries=50, **kwds):
         if is_valid_degree_sequence(seq):
             return seq
         tries+=1
-    raise networkx.NetworkXError, \
-          "Exceeded max (%d) attempts at a valid sequence."%max_tries
+    raise nx.NetworkXError(\
+          "Exceeded max (%d) attempts at a valid sequence."%max_tries)
 
 def double_edge_swap(G, nswap=1):
     """Attempt nswap double-edge swaps on the graph G.
@@ -369,17 +655,16 @@ def double_edge_swap(G, nswap=1):
     # choosing nonuniformly from the set nodes (probability weighted by degree)
     n=0
     swapcount=0
-    deg=G.degree(with_labels=True)
+    deg=G.degree()
     dk=deg.keys() # key labels 
-    cdf=networkx.utils.cumulative_distribution(deg.values())  # cdf of degree
+    cdf=nx.utils.cumulative_distribution(deg.values())  # cdf of degree
     if len(cdf)<4:
-        raise networkx.NetworkXError, \
-          "Graph has less than four nodes."
+        raise nx.NetworkXError("Graph has less than four nodes.")
     while n < nswap:
 #        if random.random() < 0.5: continue # trick to avoid periodicities?
         # pick two randon edges without creating edge list
         # choose source node indices from discrete distribution
-        (ui,xi)=networkx.utils.discrete_sequence(2,cdistribution=cdf) 
+        (ui,xi)=nx.utils.discrete_sequence(2,cdistribution=cdf) 
         if ui==xi: continue # same source, skip
         u=dk[ui] # convert index to label
         x=dk[xi] 
@@ -389,8 +674,8 @@ def double_edge_swap(G, nswap=1):
         if (x not in G[u]) and (y not in G[v]):
             G.add_edge(u,x)
             G.add_edge(v,y)
-            G.delete_edge(u,v)
-            G.delete_edge(x,y)
+            G.remove_edge(u,v)
+            G.remove_edge(x,y)
             swapcount+=1
         n+=1
     return swapcount
@@ -398,9 +683,11 @@ def double_edge_swap(G, nswap=1):
 def connected_double_edge_swap(G, nswap=1):
     """Attempt nswap double-edge swaps on the graph G.
 
-    Returns count of successful swaps.  Enforces connectivity.
+    Returns the count of successful swaps.  Enforces connectivity.
     The graph G is modified in place.
 
+    Notes
+    -----
     A double-edge swap removes two randomly choseen edges u-v and x-y
     and creates the new edges u-x and v-y::
 
@@ -414,71 +701,67 @@ def connected_double_edge_swap(G, nswap=1):
 
     The initial graph G must be connected and the resulting graph is connected.
 
-    Reference::
-
-     @misc{gkantsidis-03-markov,
-      author = "C. Gkantsidis and M. Mihail and E. Zegura",
-      title = "The Markov chain simulation method for generating connected
-               power law random graphs",
-      year = "2003",
-      url = "http://citeseer.ist.psu.edu/gkantsidis03markov.html"
-     }
-
-
+    References
+    ----------
+    .. [1] C. Gkantsidis and M. Mihail and E. Zegura,
+           The Markov chain simulation method for generating connected
+           power law random graphs, 2003.
+           http://citeseer.ist.psu.edu/gkantsidis03markov.html
     """
     import math
-    if not networkx.is_connected(G):
-       raise networkx.NetworkXException("Graph not connected")
+    if not nx.is_connected(G):
+       raise nx.NetworkXException("Graph not connected")
 
     n=0
     swapcount=0
-    deg=G.degree(with_labels=True)
-    dk=deg.keys() # key labels 
-    ideg=G.degree()
-    cdf=networkx.utils.cumulative_distribution(G.degree()) 
+    deg=G.degree()
+    dk=deg.keys() # Label key for nodes
+    ideg=G.degree().values()
+    cdf=nx.utils.cumulative_distribution(G.degree().values()) 
     if len(cdf)<4:
-        raise networkx.NetworkXError, \
-          "Graph has less than four nodes."
+        raise nx.NetworkXError("Graph has less than four nodes.")
     window=1
     while n < nswap:
         wcount=0
         swapped=[]
         while wcount < window and n < nswap:
-            # pick two randon edges without creating edge list
-            # chose source nodes from discrete distribution
-            (ui,xi)=networkx.utils.discrete_sequence(2,cdistribution=cdf) 
+            # Pick two random edges without creating edge list
+            # Choose source nodes from discrete degree distribution
+            (ui,xi)=nx.utils.discrete_sequence(2,cdistribution=cdf) 
             if ui==xi: continue # same source, skip
             u=dk[ui] # convert index to label
             x=dk[xi] 
-            v=random.choice(G.neighbors(u)) # choose target uniformly from nbrs
-            y=random.choice(G.neighbors(x)) # Note: dan't use G[u] because choice can't use dict 
+            # Choose targets uniformly from neighbors
+            v=random.choice(G.neighbors(u)) 
+            y=random.choice(G.neighbors(x)) #
             if v==y: continue # same target, skip
             if (not G.has_edge(u,x)) and (not G.has_edge(v,y)):
-                G.delete_edge(u,v)
-                G.delete_edge(x,y)
+                G.remove_edge(u,v)
+                G.remove_edge(x,y)
                 G.add_edge(u,x)
                 G.add_edge(v,y)
                 swapped.append((u,v,x,y))
                 swapcount+=1
             n+=1
             wcount+=1
-        if networkx.is_connected(G): # increase window
+        if nx.is_connected(G):
             window+=1
-        else: # undo changes from previous window, decrease window
+        else:
+            # not connected, undo changes from previous window, decrease window
             while swapped:
                 (u,v,x,y)=swapped.pop()
                 G.add_edge(u,v)
                 G.add_edge(x,y)
-                G.delete_edge(u,x)
-                G.delete_edge(v,y)
+                G.remove_edge(u,x)
+                G.remove_edge(v,y)
                 swapcount-=1
             window = int(math.ceil(float(window)/2))
-        assert G.degree() == ideg
+        assert G.degree().values() == ideg
     return swapcount
 
 
 
-def li_smax_graph(degree_seq):
+def li_smax_graph(degree_seq, create_using=None):
     """Generates a graph based with a given degree sequence and maximizing
     the s-metric.  Experimental implementation.
 
@@ -543,13 +826,16 @@ def li_smax_graph(degree_seq):
     Several optimizations are included in this code and it may be hard to read.
     Commented code to come.
     """
-    
     if not is_valid_degree_sequence(degree_seq):
-        raise networkx.NetworkXError, 'Invalid degree sequence'
+        raise nx.NetworkXError('Invalid degree sequence')
+    if create_using is not None and create_using.is_directed():
+        raise nx.NetworkXError("Directed Graph not supported")
+
     degree_seq.sort() # make sure it's sorted
     degree_seq.reverse()
     degrees_left = degree_seq[:]
-    A_graph = networkx.Graph()
+
+    A_graph = empty_graph(0,create_using)
     A_graph.add_node(0)
     a_list = [False]*len(degree_seq)
     b_set = set(range(1,len(degree_seq)))
@@ -625,7 +911,8 @@ def li_smax_graph(degree_seq):
                 #print "removing because tree condition    "
                 continue
             elif db < 2*bsize -wa:
-                raise networkx.NetworkXError, "THIS SHOULD NOT HAPPEN! - not graphable"
+                raise nx.NetworkXError(\
+                        "THIS SHOULD NOT HAPPEN!-not graphable")
                 continue
             elif wa == 2 and bsize > 0:
                 #print "removing because disconnected  cluster"
@@ -652,41 +939,23 @@ def li_smax_graph(degree_seq):
                                         -degrees_left[j], (i,j)])
     return A_graph 
 
-def connected_smax_graph(degree_seq):
+def connected_smax_graph(degree_seq, create_using=None):
     """
     Not implemented.
     """
     # incomplete implementation
     
     if not is_valid_degree_sequence(degree_seq):
-        raise networkx.NetworkXError, 'Invalid degree sequence'
+        raise nx.NetworkXError('Invalid degree sequence')
+    if create_using is not None and create_using.is_directed():
+        raise nx.NetworkXError("Directed Graph not supported")
    
     # build dictionary of node id and degree, sorted by degree, largest first
     degree_seq.sort() 
     degree_seq.reverse()
     ddict=dict(zip(xrange(len(degree_seq)),degree_seq))
 
-    G=empty_graph(1) # start with single node
+    G=empty_graph(1,create_using) # start with single node
 
     return False
 
-def s_metric(G):
-    """
-    Return the "s-Metric" of graph G:
-    the sum of the product deg(u)*deg(v) for every edge u-v in G
-    
-    Reference::
-
-     @unpublished{li-2005,
-      author = {Lun Li and David Alderson and
-               John C. Doyle and Walter Willinger},
-      title = {Towards a Theory of Scale-Free Graphs:
-               Definition, Properties, and  Implications (Extended Version)},
-      url = {http://arxiv.org/abs/cond-mat/0501169},
-      year = {2005}
-      }
-
-    """
-    # this function doesn't belong in this module
-    return sum([G.degree(u)*G.degree(v) for (u,v) in G.edges_iter()])
-   
