@@ -26,25 +26,19 @@ Useful for connected or unconnected graphs with or without edge data.
 
 """
 __author__ = """Aric Hagberg (hagberg@lanl.gov)\nDan Schult (dschult@colgate.edu)"""
-#    Copyright (C) 2004-2008 by 
+#    Copyright (C) 2004-2009 by 
 #    Aric Hagberg <hagberg@lanl.gov>
 #    Dan Schult <dschult@colgate.edu>
 #    Pieter Swart <swart@lanl.gov>
-#    Distributed under the terms of the GNU Lesser General Public License
-#    http://www.gnu.org/copyleft/lesser.html
+#    All rights reserved.
+#    BSD license.
 
 __all__ = ['read_multiline_adjlist', 'write_multiline_adjlist',
            'read_adjlist', 'write_adjlist']
 
-import cPickle 
-import codecs
-import locale
-import string
-import sys
-import time
 
 from networkx.utils import is_string_like,_get_fh
-import networkx
+import networkx as nx
 
 def write_multiline_adjlist(G, path, delimiter=' ', comments='#'):
     """
@@ -78,8 +72,11 @@ def write_multiline_adjlist(G, path, delimiter=' ', comments='#'):
     >>> nx.write_multiline_adjlist(G,fh)
 
     """
+    import sys
+    import time
+
     fh=_get_fh(path,mode='w')        
-    pargs=comments+" "+string.join(sys.argv,' ')
+    pargs=comments+" ".join(sys.argv)
     fh.write("%s\n" % (pargs))
     fh.write(comments+" GMT %s\n" % (time.asctime(time.gmtime())))
     fh.write(comments+" %s\n" % (G.name))
@@ -88,10 +85,12 @@ def write_multiline_adjlist(G, path, delimiter=' ', comments='#'):
         if is_string_like(t): return t
         return str(t)
 
-    if G.directed:
-        if G.multigraph:
+    if G.is_directed():
+        if G.is_multigraph():
             for s,nbrs in G.adjacency_iter():
-                nbr_edges=[ (u,d) for u,dl in nbrs.iteritems() for d in dl]
+                nbr_edges=[ (u,data) 
+                            for u,datadict in nbrs.iteritems() 
+                            for key,data in datadict.items()]
                 deg=len(nbr_edges)
                 fh.write(make_str(s)+delimiter+"%i\n"%(deg))
                 for u,d in nbr_edges:
@@ -108,11 +107,14 @@ def write_multiline_adjlist(G, path, delimiter=' ', comments='#'):
                         fh.write(make_str(u)+'\n')
                     else:   
                         fh.write(make_str(u)+delimiter+make_str(d)+"\n")
-    else: #undirected
-        if G.multigraph:
-            seen={}  # helper dict used to avoid duplicate edges
+    else: # undirected
+        if G.is_multigraph():
+            seen=set()  # helper dict used to avoid duplicate edges
             for s,nbrs in G.adjacency_iter():
-                nbr_edges=[ (u,d) for u,dl in nbrs.iteritems() if u not in seen for d in dl]
+                nbr_edges=[ (u,data) 
+                            for u,datadict in nbrs.iteritems() 
+                            if u not in seen
+                            for key,data in datadict.items()]
                 deg=len(nbr_edges)
                 fh.write(make_str(s)+delimiter+"%i\n"%(deg))
                 for u,d in nbr_edges:
@@ -120,9 +122,9 @@ def write_multiline_adjlist(G, path, delimiter=' ', comments='#'):
                         fh.write(make_str(u)+'\n')
                     else:   
                         fh.write(make_str(u)+delimiter+make_str(d)+"\n")
-                seen[s]=1
+                seen.add(s)
         else: # undirected single edges
-            seen={}  # helper dict used to avoid duplicate edges
+            seen=set()  # helper dict used to avoid duplicate edges
             for s,nbrs in G.adjacency_iter():
                 nbr_edges=[ (u,d) for u,d in nbrs.iteritems() if u not in seen]
                 deg=len(nbr_edges)
@@ -132,7 +134,7 @@ def write_multiline_adjlist(G, path, delimiter=' ', comments='#'):
                         fh.write(make_str(u)+'\n')
                     else:   
                         fh.write(make_str(u)+delimiter+make_str(d)+"\n")
-                seen[s]=1
+                seen.add(s)
             
 def read_multiline_adjlist(path, comments="#", delimiter=' ',
                            create_using=None,
@@ -169,7 +171,7 @@ def read_multiline_adjlist(path, comments="#", delimiter=' ',
 
     edgetype is a function to convert edge data strings to edgetype
 
-    >>> G=nx.read_multiline_adjlist("test.adjlist", edgetype=int)
+    >>> G=nx.read_multiline_adjlist("test.adjlist")
 
     create_using is an optional networkx graph type, the default is
     Graph(), a simple undirected graph 
@@ -195,7 +197,7 @@ def read_multiline_adjlist(path, comments="#", delimiter=' ',
      d 1
      e
 
-     Wiht edge data::
+     With edge data::
 
      # source target for XGraph or XDiGraph with edge data
      a 2
@@ -213,8 +215,14 @@ def read_multiline_adjlist(path, comments="#", delimiter=' ',
     >>> fh=codecs.open("test.adjlist",'r',encoding='utf=8') # utf-8 encoding
     >>> G=nx.read_multiline_adjlist(fh)
     """
+    try:
+        from ast import literal_eval 
+    except:
+        literal_eval=eval
+        pass # use potentially unsafe built-in eval 
+
     if create_using is None:
-        G=networkx.Graph()
+        G=nx.Graph()
     else:
         try:
             G=create_using
@@ -225,13 +233,12 @@ def read_multiline_adjlist(path, comments="#", delimiter=' ',
     inp=_get_fh(path)        
 
     for line in inp:
-#        if line.startswith("#") or line.startswith("\n"):
-#            continue
-#        line=line.strip() #remove trailing \n 
-        line = line[:line.find(comments)].strip()
+        p=line.find(comments)
+        if p>=0:
+            line = line[:line.find(comments)]
         if not line: continue
         try:
-            (u,deg)=line.split(delimiter)
+            (u,deg)=line.strip().split(delimiter)
             deg=int(deg)
         except:
             raise TypeError("Failed to read node and degree on line (%s)"%line)
@@ -243,31 +250,42 @@ def read_multiline_adjlist(path, comments="#", delimiter=' ',
                       %(u,nodetype))
         G.add_node(u)
         for i in range(deg):
-            line=inp.next().strip()
-            vlist=line.split(delimiter)
+            while True:
+                try:
+                    line = inp.next()
+                except StopIteration:
+                    msg = "Failed to find neighbor for node (%s)" % (u,)
+                    raise TypeError(msg)
+                p=line.find(comments)
+                if p>=0:
+                    line = line[:line.find(comments)]
+                if line: break
+            vlist=line.strip().split(delimiter)
             numb=len(vlist)
-            if numb>0:
-                v=vlist[0]
-                if nodetype is not None:
-                    try:
-                        v=nodetype(v)
-                    except:
-                        raise TypeError("Failed to convert node (%s) to type %s"\
-                                        %(v,nodetype))
-            if numb==1:
-                G.add_edge(u,v)
-            elif numb==2:
-                d=vlist[1]
-                if edgetype is not None:
-                    try:
-                        d=edgetype(d)
-                    except:
-                        raise TypeError\
-                              ("Failed to convert edge data (%s) to type %s"\
-                                %(d, edgetype))
-                G.add_edge(u,v,d)
+            if numb<1:
+                continue # isolated node
+            v=vlist.pop(0)
+            data=''.join(vlist)
+            if nodetype is not None:
+                try:
+                    v=nodetype(v)
+                except:
+                    raise \
+                        TypeError("Failed to convert node (%s) to type %s"\
+                                       %(v,nodetype))
+            if edgetype is not None:
+                try:
+                    edgedata={'weight':edgetype(data)}
+                except:
+                    raise TypeError("Failed to convert edge data (%s) to type %s"\
+                                    %(data, edgetype))
             else:
-                raise TypeError("Failed to read line: %s"%vlist)
+                try: # try to evaluate 
+                    edgedata=literal_eval(data)
+                except:
+                    edgedata={}
+            G.add_edge(u,v,**edgedata)  
+
     return G
 
 
@@ -304,8 +322,10 @@ def write_adjlist(G, path, comments="#", delimiter=' '):
     Does not handle edge data. 
     Use 'write_edgelist' or 'write_multiline_adjlist'
     """
+    import sys
+    import time
     fh=_get_fh(path,mode='w')        
-    pargs=comments+" "+string.join(sys.argv,' ')
+    pargs=comments+" ".join(sys.argv)
     fh.write("%s\n" % (pargs))
     fh.write(comments+" GMT %s\n" % (time.asctime(time.gmtime())))
     fh.write(comments+" %s\n" % (G.name))
@@ -313,21 +333,22 @@ def write_adjlist(G, path, comments="#", delimiter=' '):
     def make_str(t):
         if is_string_like(t): return t
         return str(t)
-    directed=G.directed
+    directed=G.is_directed()
 
-    seen={}
+    seen=set()
     for s,nbrs in G.adjacency_iter():
         fh.write(make_str(s)+delimiter)
         for t,data in nbrs.iteritems():
-            if not directed and t in seen: continue
-            if G.multigraph:
-                for d in data:
+            if not directed and t in seen: 
+                continue
+            if G.is_multigraph():
+                for d in data.values():
                     fh.write(make_str(t)+delimiter)
             else:
                 fh.write(make_str(t)+delimiter)
         fh.write("\n")            
         if not directed: 
-            seen[s]=1
+            seen.add(s)
 
 
 def read_adjlist(path, comments="#", delimiter=' ',
@@ -384,7 +405,7 @@ def read_adjlist(path, comments="#", delimiter=' ',
 
     """
     if create_using is None:
-        G=networkx.Graph()
+        G=nx.Graph()
     else:
         try:
             G=create_using
@@ -395,12 +416,11 @@ def read_adjlist(path, comments="#", delimiter=' ',
     fh=_get_fh(path)        
 
     for line in fh.readlines():
-        line = line[:line.find(comments)].strip()
+        p=line.find(comments)
+        if p>=0:
+            line = line[:line.find(comments)]
         if not len(line): continue
-#        if line.startswith("#") or line.startswith("\n"):
-#            continue
-#        line=line.strip() #remove trailing \n 
-        vlist=line.split(delimiter)
+        vlist=line.strip().split(delimiter)
         u=vlist.pop(0)
         # convert types
         if nodetype is not None:
@@ -410,11 +430,12 @@ def read_adjlist(path, comments="#", delimiter=' ',
                 raise TypeError("Failed to convert node (%s) to type %s"\
                                 %(u,nodetype))
         G.add_node(u)
-        try:
-            vlist=map(nodetype,vlist)
-        except:
-            raise TypeError("Failed to convert nodes (%s) to type %s"\
-                            %(','.join(vlist),nodetype))
+        if nodetype is not None:
+            try:
+                vlist=map(nodetype,vlist)
+            except:
+                raise TypeError("Failed to convert nodes (%s) to type %s"\
+                                    %(','.join(vlist),nodetype))
         for v in vlist:
             G.add_edge(u,v)
     return G
