@@ -4,20 +4,51 @@
 #    Aric Hagberg <hagberg@lanl.gov>
 #    Dan Schult <dschult@colgate.edu>
 #    Pieter Swart <swart@lanl.gov>
-#    Distributed under the terms of the GNU Lesser General Public License
-#    http://www.gnu.org/copyleft/lesser.html
+#    All rights reserved.
+#    BSD license.
 
 
 import os
 import re
 
 
+def write_versionfile():
+    """Creates a file containing version information."""
+    base = os.path.split(__file__)[0]
+    versionfile = os.path.join(base, 'version.py')
+    if revision is None and os.path.isfile(versionfile):
+        # Unable to get revision info, so probably not in an SVN directory
+        # If a version.py already exists, let's not overwrite it.
+        # Useful mostly for nightly tarballs.
+        return
+    fh = open(versionfile, 'w')
+    text = '''"""
+Version information for NetworkX, created during installation.
+
+Do not add this file to the repository.
+
+"""
+
+__version__ = '%(version)s'
+__revision__ = %(revision)s
+__date__ = '%(date)s'
+
+'''
+    if revision is not None:
+        rev = "'%s'" % (revision,)
+    else:
+        rev = revision
+    subs = {'version': version,
+            'revision': rev,
+            'date': date}
+    fh.write(text % subs)
+    fh.close()
+
 def get_svn_revision():
-    #import networkx
     rev = None
-    path ="."
-    entries_path = '%s/.svn/entries' % path
-    if os.path.exists(entries_path):
+    base = os.path.split(__file__)[0]
+    entries_path = os.path.join(base, '.svn', 'entries')
+    if os.path.isfile(entries_path):
         entries = open(entries_path, 'r').read()
         # Versions >= 7 of the entries file are flat text.  The first line is
         # the version number. The next set of digits after 'dir' is the revision.
@@ -26,15 +57,24 @@ def get_svn_revision():
             if rev_match:
                 rev = rev_match.groups()[0]
     if rev:
-        return 'dev%s' % rev
-    return None
+        return rev
+    else:
+        return None
 
 
 name = 'networkx'
-version = '0.99'
+version = '1.1'
+
+# Declare current release as a development release.
+# Change to False before tagging a release; then change back.
+dev = False
+
 revision = None
-if revision is not None:
-    version+=".%s"%revision
+if dev:
+    version += '.dev'   
+    revision = get_svn_revision()
+    if revision is not None:
+        version += "%s" % revision
 
 description = "Python package for creating and manipulating graphs and networks"
 
@@ -44,20 +84,22 @@ NetworkX is a Python package for the creation, manipulation, and
 study of the structure, dynamics, and functions of complex networks.  
 
 """
-license = 'LGPL'
+license = 'BSD'
 authors = {'Hagberg' : ('Aric Hagberg','hagberg@lanl.gov'),
            'Schult' : ('Dan Schult','dschult@colgate.edu'),
            'Swart' : ('Pieter Swart','swart@lanl.gov')
            }
+maintainer = "NetworkX Developers",
+maintainer_email = "networkx-discuss@googlegroups.com",
 url = 'http://networkx.lanl.gov/'
 download_url="http://networkx.lanl.gov/download/networkx"
-platforms = ['Linux','Mac OSX','Windows XP/2000/NT']
+platforms = ['Linux','Mac OSX','Windows','Unix']
 keywords = ['Networks', 'Graph Theory', 'Mathematics', 'network', 'graph', 'discrete mathematics', 'math']
 classifiers = [
         'Development Status :: 4 - Beta',
         'Intended Audience :: Developers',
         'Intended Audience :: Science/Research',
-        'License :: OSI Approved :: GNU Library or Lesser General Public License (LGPL)',
+        'License :: OSI Approved :: BSD License',
         'Operating System :: OS Independent',
         'Programming Language :: Python',
         'Topic :: Software Development :: Libraries :: Python Modules',
@@ -72,3 +114,7 @@ import time
 date = time.asctime()
 del time
 
+if __name__ == '__main__':
+    # Write versionfile for nightly snapshots.
+    write_versionfile()
+
